import os

import common as cm
from util import rmfile
from osutil import find_rexp, run
from pylatexenc.latex2text import LatexNodes2Text


def latex_to_text(latex_str):
    return LatexNodes2Text().latex_to_text(latex_str).encode('utf8')

def has_class(main):
    '''Return boolean on whether `main` defines class document.'''
    # cm.logger.debug(main)
    try:
        match = find_rexp(cm.DOCCLASS_REXP, main)
    except UnicodeDecodeError:
        return False
    return bool(match is not None)


def get_num_pages(log_output):
    '''Parse the number of pages from the log file.'''
    matches = cm.OUT_PAGES_REXP.search(log_output)
    if matches is None:
        raise cm.JobError('number of pages of pdf not found')
    num_pages = int(matches.group(1))
    cm.logger.debug('Found %s pages in pdf' % num_pages)
    return num_pages


def find_bib_backend(main):
    '''Find the backend use to compile bib references.'''
    for line in open(main):
        for key in cm.BIBL_BACKENDS:
            if key in line:
                return cm.BIBL_BACKENDS[key]
    raise cm.JobError('biblio backend not found')


def remove_aux_files(main):
    '''Remove all auxiliary and temporary files generated in compilation.'''
    for ext in ['aux', 'out', 'log', 'idx', 'pdf']:
        rmfile(main[:-3] + ext)


def compile_source(main, refs=True):
    '''Compile source tex.

    Infers the backend for reference management and compiles
    with the combination: pdflatex + bib + pdflatex + pdflatex
    '''
    if refs:
        backend = find_bib_backend('%s.tex' % main)
        bibl_cmd = (cm.BIBL_CMD % backend).format(main)

    # build command
    latex_cmd = cm.LATEX_CMD.format(main)

    # run pdflatex + bib + pdflatex + pdflatex
    cm.logger.debug(latex_cmd)
    output = run(latex_cmd)
    if refs and backend != 'native':
        cm.logger.debug(bibl_cmd)
        output = run(bibl_cmd)
        output = run(latex_cmd)
    output = run(latex_cmd)

    return output


def find_candidates(path):
    '''Return list of files that define the document class.'''
    main_candidates = []
    for root, _, fnames in os.walk(path):
        for fname in fnames:
            name, extension = os.path.splitext(fname)
            if extension != '.tex':
                continue
            if has_class(os.path.join(root, fname)):
                main_candidates.append(os.path.join(root, fname))
            if name == 'main':
                return [os.path.join(root, fname)]
    return main_candidates

def find_main(path):
    '''Return the best candidate as main file.'''
    if os.path.isfile(os.path.join(path, 'main.tex')):
        return os.path.join(path, 'main.tex')
    main_candidates = find_candidates(path)
    cm.logger.debug('files with class %s' % main_candidates)
    if len(main_candidates) == 1:
        return main_candidates[0]
    elif len(main_candidates) > 1:
        for candidate in main_candidates:
            # assumption: return first one that matches
            for name in cm.MAIN_NAME:
                if name in os.path.basename(candidate).lower():
                    return candidate
