import os
import re
from contextlib import contextmanager
from subprocess import Popen, STDOUT, PIPE


def find_rexp(rexp, file_path):
    '''Return first matching group for `rexp`.'''
    for line in open(file_path):
        matches = rexp.search(line)
        if matches is not None:
            return matches.groups(1)


@contextmanager
def cd(newdir):
    '''See: http://stackoverflow.com/a/24176022'''
    prevdir = os.getcwd()
    os.chdir(os.path.expanduser(newdir))
    try:
        yield
    finally:
        os.chdir(prevdir)


def getstatusoutput(cmd):
    '''See: http://stackoverflow.com/a/11346017'''
    p = Popen(cmd.split(), stderr=STDOUT, stdout=PIPE)
    stdout, stderr = p.communicate()
    return p.returncode, stdout


def run(cmd):
    '''Run the command `cmd`.'''
    status, stdout = getstatusoutput(cmd)
    if status != 0:
        raise SystemError(stdout)
    return stdout


def read_csv(file_path, delimiter=','):
    '''Iterate over a csv file.'''
    import csv
    with open(file_path) as fin:
        for row in csv.DictReader(fin, delimiter=delimiter):
            yield row
